unit hiDatePicker; { ver 3.00}

interface

uses Windows,Messages,Kol,Share,Win;

const
   ICC_DATE_CLASSES     = $00000100;

   DTS_SHORTDATEFORMAT  = $0000;
   DTS_UPDOWN           = $0001;
   DTS_SHOWNONE         = $0002;
   DTS_LONGDATEFORMAT   = $0004;
   DTS_TIMEFORMAT       = $0009;
   DTS_APPCANPARSE      = $0010;
   DTS_RIGHTALIGN       = $0020;
   //Notification codes
   DTN_FIRST            = 0-760;
   DTN_DATETIMECHANGE   = DTN_FIRST + 1;
   DTN_DROPDOWN         = DTN_FIRST + 6;
   DTN_CLOSEUP          = DTN_FIRST + 7;
   //Messages
   DTM_FIRST            = $1000;
   DTM_GETSYSTEMTIME    = DTM_FIRST + 1;
   DTM_SETSYSTEMTIME    = DTM_FIRST + 2;
   DTM_SETFORMATA       = DTM_FIRST + 5;
   DTM_SETMCCOLOR       = DTM_FIRST + 6;
   DTM_GETMCCOLOR       = DTM_FIRST + 7;
   DTM_GETMONTHCAL      = DTM_FIRST + 8;
   DTM_SETMCFONT        = DTM_FIRST + 9;   

   GDT_VALID            = 0;
   GDT_NONE             = 1;
   //Color schemma
   MCSC_BACKGROUND   =  0;   // the background color (between months)
   MCSC_TEXT         =  1;   // the dates
   MCSC_TITLEBK      =  2;   // background of the title
   MCSC_TITLETEXT    =  3;
   MCSC_MONTHBK      =  4;   // background within the month cal
   MCSC_TRAILINGTEXT =  5;   // the text color of header & trailing days

type
TColorOption  = (clBackground,clText,clTitleBk,clTitleText,clMonthBk,clTrailing);
TPickerOption = (piShortDate,piUpDown,piCheckBox,piLongDate,piTime,piAppCanParse,piRightAlign);
TPickerOptions = Set of TPickerOption;

type

INITCOMMONCONTROLEX = packed record
    dwSize: DWORD;             // size of this structure
    dwICC: DWORD;              // flags indicating which classes to be initialized
end;

NMHDR = packed record
   hwndFrom: HWND;
   idFrom: UINT;
   code: Integer;
end;

TNMHdr = NMHDR;
PNMHdr = ^TNMHdr;

NMDATETIMECHANGE = packed record
   nmhdr: TNmHdr;
   dwFlags: DWORD;         // GDT_VALID or GDT_NONE
   st: TSystemTime;        // valid iff dwFlags = GDT_VALID
end;

PNMDateTimeChange = ^TNMDateTimeChange;
TNMDateTimeChange = NMDATETIMECHANGE;

TInitCommonControlsEx = INITCOMMONCONTROLEX;

const  PickerFlags : array[TPickerOption] of Integer = (DTS_SHORTDATEFORMAT,DTS_UPDOWN,DTS_SHOWNONE,DTS_LONGDATEFORMAT,DTS_TIMEFORMAT,DTS_APPCANPARSE,DTS_RIGHTALIGN);
const ColorFlags : array[TColorOption] of integer = (MCSC_BACKGROUND,MCSC_TEXT,
MCSC_TITLEBK,MCSC_TITLETEXT,MCSC_MONTHBK,MCSC_TRAILINGTEXT);

type
 THIDatePicker = class(THIWin)
   private
     dt: TDateTime;
     fFormat : string;
     fOnChange : TOnEvent;
     procedure _OnChange(Sender:PObj);
     function  GetDateTime : TDateTime;
     procedure SetDateTime(const Value : TDateTime);
     procedure SetFormat(const Value : String);
     procedure SetColors(const ColorOption : TColorOption; const Value : TColor);
     function GetColors(const ColorOption : TColorOption) : TColor;

   public
     _prop_Style:byte;
     _prop_AppCanParse:boolean;
     _prop_AlignPicker:byte;
     _prop_BackgroundColor:TColor;
     _prop_TitleBkColor:TColor;
     _prop_TitleTextColor:TColor;
     _prop_MonthBkColor:TColor;
     _prop_TrailingColor:TColor;
     _prop_DateFormat:integer;
     _prop_DateMode:integer;

     _prop_Time:boolean;
     _prop_Visible:boolean;
     _prop_Enabled:boolean;
     _prop_Format:string;

     _event_OnChange:THI_Event;

     property DateTime : TDateTime read GetDateTime write SetDateTime;
     property Format : String read fFormat write SetFormat;
     property PColor[const ColorOption : TColorOption]: TColor read GetColors write SetColors;
     property OnChange : TOnEvent read fOnChange write fOnChange;

     destructor Destroy; override;
     procedure Init; override;
     procedure _work_doSetDate(var _Data:TData; Index:word);
     procedure _var_Date(var _Data:TData; Index:word);
     procedure _var_Time(var _Data:TData; Index:word);
     procedure _var_FormatTime(var _Data:TData; Index:word);
end;

function TimeToStr(const Format:string; const t:TSystemTime):string;
function HiAsmFont2KolFont(_f: TFontRec): PGraphicTool;

implementation

function TimeToStr(const Format:string; const t:TSystemTime):string;
const namstr:string = 'YMWDhms';
type TTimeValue = array[0..6] of WORD;
     PTimeValue = ^TTimeValue;
var
  i:byte;
  function TwoDigit(value:integer):string;
  begin
    if Value < 10 then
      Result := '0' + Int2Str(value)
    else Result := Int2Str(value);
  end;
begin
   Result := Format;
   for i := 0 to 6 do
     Replace(Result,namstr[i+1],TwoDigit(PTimeValue(@t)^[i]));
end;

function HiAsmFont2KolFont(_f:TFontRec):PGraphicTool;
begin
   Result:= NewFont;
   Result.Color:= _f.Color;
   Share.SetFont(Result,_f.Style);
   Result.FontName:= _f.Name;
   Result.FontHeight:= _hi_SizeFnt(_f.Size);
   Result.FontCharset:= _f.CharSet;
end;

function InitCommonControlsEx(var ICC: TInitCommonControlsEx) : Bool;stdcall;external 'comctl32.dll';

function PickerWndProc(Sender : PControl;var Msg:TMsg;var Rslt:Integer):Boolean;
var   NMDC: PNMDateTimeChange;
begin
Result := False;
if Msg.message = WM_NOTIFY then begin
    NMDC := PNMDateTimeChange( Msg.lParam );
    case  NMDC.nmhdr.code of
       DTN_DATETIMECHANGE: begin
          if NMDC.dwFlags = GDT_VALID then begin
             if Assigned(Sender.OnChange) then Sender.OnChange(Sender);
             Result := True;
          end;
       end;
   end;
end;
end;

procedure THIDatePicker.Init;
var   opt: TPickerOptions;
      Flags: Integer;
      icex: INITCOMMONCONTROLEX;
begin
   opt:=[];
// 
   if _prop_DateFormat=0 then include(opt,piShortDate)
   else include(opt,piLongDate);
   if _prop_AppCanParse then include(opt,piAppCanParse);
   if _prop_AlignPicker = 1 then include(opt,piRightAlign);
   if _prop_DateMode = 1 then include(opt,piUpDown);
   if _prop_Time then include(opt,piTime);
// 
   icex.dwSize := sizeof(INITCOMMONCONTROLEX);
   icex.dwICC := ICC_DATE_CLASSES;
   InitCommonControlsEx(icex);
   Flags := MakeFlags(@opt,PickerFlags);
   Control:= _NewCommonControl(FParent,'SysDateTimePick32',
            WS_VISIBLE or WS_CHILD or WS_TABSTOP or Flags,True,nil);
   InitCommonControlCommonNotify(Control);
   Control.ExStyle:= Control.ExStyle or WS_EX_CLIENTEDGE;
   Control.Tabstop := True;
   Control.AttachProc(PickerWndProc);
   Control.OnChange:= _OnChange;

inherited;

   Control.Align:= _prop_Align; 
   Control.Left:= _prop_Left;
   Control.Top:= _prop_Top;    
   Control.Width:= _prop_Width;
   Control.Height:= _prop_Height;    
   Control.Cursor:= _prop_Cursor;
   Control.Font.Assign(HiAsmFont2KolFont(_prop_Font));
   SetHint(_prop_Hint);
   
   if _prop_Style = 0 then exit;
   
   PColor[clBackground]:= _prop_Color;
   PColor[clTitleBk]:= _prop_TitleBkColor;
   PColor[clText]:= Control.Font.Color;
   PColor[clTitleText]:= _prop_TitleTextColor;
   PColor[clMonthBk]:= _prop_MonthBkColor;
   PColor[clTrailing]:= _prop_TrailingColor;

end;

destructor THIDatePicker.Destroy;
begin
  inherited Destroy;
end;

function THIDatePicker.GetDateTime;
var   st : TSystemTime;
begin
   Control.Perform(DTM_GETSYSTEMTIME,0,Longint(@st));
   SystemTime2DateTime(st,dt);
   Result := dt;
end;

procedure THIDatePicker.SetDateTime;
var   st : TSystemTime;
begin
   dt := Value;
   DateTime2SystemTime(dt,st);
   Control.Perform(DTM_SETSYSTEMTIME,GDT_VALID,Longint(@st));
end;

procedure THIDatePicker.SetFormat;
begin
   if fFormat <> Value then begin
      fFormat := Value;
      Control.Perform(DTM_SETFORMATA, 0, Longint(PChar(Value)));
   end;
end;

procedure THIDatePicker.SetColors;
begin
   Control.Perform(DTM_SETMCCOLOR,ColorFlags[ColorOption], Value);
end;

function THIDatePicker.GetColors;
begin
   Result:= Control.Perform(DTM_GETMCCOLOR, ColorFlags[ColorOption], 0);
end;

procedure THIDatePicker._OnChange;
begin
   _hi_OnEvent(_event_onChange);
end;

procedure THIDatePicker._var_Date;
var   s:string;
begin
   s:=DateTime2StrShort(DateTime);
   dtString(_Data,GetTok(s,' '));
end;

procedure THIDatePicker._var_Time;
var   s:string;
begin
   s:=DateTime2StrShort(DateTime);
   GetTok(s,' ');
   dtString(_Data,s);
end;

procedure THIDatePicker._work_doSetDate;
begin
   DateTime:= Str2DateTimeShort(ToString(_Data));
end;

procedure THIDatePicker._var_FormatTime;
var   SystemTime: TSystemTime;
begin
   GetLocalTime(SystemTime);
   dtString(_Data,TimeToStr(_prop_Format,SystemTime));
end;

end.
